from lion_core.protocols.adapter import AdapterRegistry
from lion_core.protocols.data_adapter import JsonDataAdapter
from lion_core.protocols.data_source_adapter import JsonFileAdapter


class ComponentAdapterRegistry(AdapterRegistry):
    pass


ComponentAdapterRegistry.register(JsonDataAdapter)
ComponentAdapterRegistry.register(JsonFileAdapter)

__all__ = ["ComponentAdapterRegistry"]



from collections.abc import Callable
from pathlib import Path

from lionfuncs import read_file, save_to_file

from lion_core.protocols.adapter import Adapter
from lion_core.protocols.data_adapter import DataAdapter, JsonDataAdapter


class DataSourceAdapterConfig:

    data_adapter: DataAdapter

    save_func: Callable
    save_config: dict = {}

    read_func: Callable
    read_config: dict = {}

    def __init__(
        self,
        data_adapter: DataAdapter,
        save_func: Callable = save_to_file,
        save_config: dict = {},
        read_func: Callable = read_file,
        read_config: dict = {},
    ):
        self.data_adapter = data_adapter
        self.save_func = save_func
        self.save_config = save_config
        self.read_func = read_func
        self.read_config = read_config


class DataSourceAdapter(Adapter):

    config: DataSourceAdapterConfig

    @classmethod
    def from_obj(cls, subj_cls, obj, **kwargs) -> dict:
        """
        kwargs for read_func
        """
        kwargs = {**cls.config.read_config, **kwargs}
        obj = cls.config.read_func(obj, **kwargs)
        return cls.config.data_adapter.from_obj(subj_cls, obj)

    @classmethod
    def to_obj(cls, subj, **kwargs) -> str:
        """
        kwargs for save_func
        """
        kwargs = {**cls.config.save_config, **kwargs}
        obj = cls.config.data_adapter.to_obj(subj)
        return cls.config.save_func(obj, **kwargs)


json_file_config = DataSourceAdapterConfig(
    data_adapter=JsonDataAdapter,
    save_func=save_to_file,
    save_config={
        "directory": Path(".") / "data" / "json_files",
        "filename": "unnamed_json_file",
        "extension": ".json",
        "timestamp": True,
        "random_hash_digits": 4,
    },
    read_func=read_file,
    read_config={},
)


class JsonFileAdapter(DataSourceAdapter):

    obj_key = "json_file"
    verbose = False
    config = json_file_config


__all__ = ["JsonFileAdapter", "DataSourceAdapter"]



from collections.abc import Callable

from lionfuncs import to_dict, to_str

from lion_core.generic.base import RealElement
from lion_core.protocols.adapter import Adapter


class DataAdapterConfig:

    from_obj_parser: Callable  # need to return a dict
    from_obj_config: dict
    to_obj_parser: Callable
    to_obj_config: dict

    def __init__(
        self, from_obj_parser, from_obj_config, to_obj_parser, to_obj_config
    ):
        self.from_obj_parser = from_obj_parser
        self.from_obj_config = from_obj_config
        self.to_obj_parser = to_obj_parser
        self.to_obj_config = to_obj_config


class DataAdapter(Adapter):

    config: DataAdapterConfig

    @classmethod
    def from_obj(cls, subj_cls: type[RealElement], obj, **kwargs) -> dict:
        kwargs = {**cls.config.from_obj_config, **kwargs}
        return cls.config.from_obj_parser(obj, **kwargs)

    @classmethod
    def to_obj(cls, subj: RealElement, **kwargs) -> str:
        subj = subj.to_dict()
        kwargs = {**cls.config.to_obj_config, **kwargs}
        return cls.config.to_obj_parser(subj, **kwargs)


json_data_config = DataAdapterConfig(
    from_obj_parser=to_dict,
    from_obj_config={
        "fuzzy_parse": False,
        "str_type": "json",
        "suppress": False,
        "recursive": True,
        "max_recursive_depth": 5,
        "recursive_python_only": True,
        "exclude_types": (),
    },
    to_obj_parser=to_str,
    to_obj_config={
        "use_model_dump": False,
        "serialize_as": "json",
        "strip_lower": False,
        "chars": None,  # additional kwargs for json.dumps
    },
)


class JsonDataAdapter(DataAdapter):

    obj_key = "json_data"
    verbose = False
    config = json_data_config


__all__ = ["JsonDataAdapter", "DataAdapter"]
